@@ -33,5 +33,5 @@
 bool base64_decode( const char *b64, const size_t b64_len,
 		    char *raw, size_t *raw_len );
 
-bool base64_encode( const char *raw, const size_t raw_len,
-		    char *b64, size_t b64_len );
+void base64_encode( const char *raw, const size_t raw_len,
+		    char *b64, const size_t b64_len );
