@@ -31,9 +31,9 @@
 */
 
 #include "config.h"
+#include "version.h"
 
 #include <stdlib.h>
-#include <string.h>
 #include <unistd.h>
 
 #include "stmclient.h"
@@ -70,15 +70,15 @@
 #  error "SysV or X/Open-compatible Curses header file required"
 #endif
 
-void usage( const char *argv0 ) {
-  fprintf( stderr, "mosh-client (%s)\n", PACKAGE_STRING );
+static void usage( const char *argv0 ) {
+  fprintf( stderr, "mosh-client (%s) [build %s]\n", PACKAGE_STRING, BUILD_VERSION );
   fprintf( stderr, "Copyright 2012 Keith Winstein <mosh-devel@mit.edu>\n" );
   fprintf( stderr, "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\nThis is free software: you are free to change and redistribute it.\nThere is NO WARRANTY, to the extent permitted by law.\n\n" );
 
   fprintf( stderr, "Usage: %s IP PORT\n       %s -c\n", argv0, argv0 );
 }
 
-void print_colorcount( void )
+static void print_colorcount( void )
 {
   /* check colors */
   setupterm((char *)0, 1, (int *)0);
@@ -93,7 +93,11 @@ void print_colorcount( void )
   printf( "%d\n", color_val );
 }
 
+#ifdef NACL
+int mosh_main( int argc, char *argv[] )
+#else
 int main( int argc, char *argv[] )
+#endif
 {
   /* For security, make sure we don't dump core */
   Crypto::disable_dumping_core();
@@ -117,7 +121,6 @@ int main( int argc, char *argv[] )
   }
 
   char *ip, *desired_port;
-  int port;
 
   if ( argc - optind != 2 ) {
     usage( argv[ 0 ] );
@@ -128,13 +131,6 @@ int main( int argc, char *argv[] )
   desired_port = argv[ optind + 1 ];
 
   /* Sanity-check arguments */
-  if ( ip
-       && ( strspn( ip, "0123456789." ) != strlen( ip ) ) ) {
-    fprintf( stderr, "%s: Bad IP address (%s)\n\n", argv[ 0 ], ip );
-    usage( argv[ 0 ] );
-    exit( 1 );
-  }
-
   if ( desired_port
        && ( strspn( desired_port, "0123456789" ) != strlen( desired_port ) ) ) {
     fprintf( stderr, "%s: Bad UDP port (%s)\n\n", argv[ 0 ], desired_port );
@@ -142,8 +138,6 @@ int main( int argc, char *argv[] )
     exit( 1 );
   }
 
-  port = myatoi( desired_port );
-
   /* Read key from environment */
   char *env_key = getenv( "MOSH_KEY" );
   if ( env_key == NULL ) {
@@ -170,28 +164,25 @@ int main( int argc, char *argv[] )
   set_native_locale();
 
   try {
-    STMClient client( ip, port, key, predict_mode );
+    STMClient client( ip, desired_port, key, predict_mode );
     client.init();
 
     try {
       client.main();
-    } catch ( const Network::NetworkException &e ) {
-      fprintf( stderr, "Network exception: %s: %s\r\n",
-	       e.function.c_str(), strerror( e.the_errno ) );
-    } catch ( const Crypto::CryptoException &e ) {
-      fprintf( stderr, "Crypto exception: %s\r\n",
-	       e.text.c_str() );
+    } catch ( ... ) {
+      client.shutdown();
+      throw;
     }
 
     client.shutdown();
-  } catch ( const Network::NetworkException& e ) {
-    fprintf( stderr, "Network exception: %s: %s\r\n",
-	     e.function.c_str(), strerror( e.the_errno ) );
-  } catch ( const Crypto::CryptoException& e ) {
+  } catch ( const Network::NetworkException &e ) {
+    fprintf( stderr, "Network exception: %s\r\n",
+	     e.what() );
+  } catch ( const Crypto::CryptoException &e ) {
     fprintf( stderr, "Crypto exception: %s\r\n",
-	     e.text.c_str() );
-  } catch ( const std::string& s ) {
-    fprintf( stderr, "Error: %s\r\n", s.c_str() );
+	     e.what() );
+  } catch ( const std::exception &e ) {
+    fprintf( stderr, "Error: %s\r\n", e.what() );
   }
 
   printf( "\n[mosh is exiting.]\n" );
