@@ -31,6 +31,7 @@
 */
 
 #include "config.h"
+#include "version.h"
 
 #include <errno.h>
 #include <locale.h>
@@ -50,8 +51,7 @@
 #include <utempter.h>
 #endif
 #include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#include <netdb.h>
 #include <time.h>
 #include <sys/stat.h>
 
@@ -91,28 +91,28 @@
 
 typedef Network::Transport< Terminal::Complete, Network::UserStream > ServerConnection;
 
-void serve( int host_fd,
-	    Terminal::Complete &terminal,
-	    ServerConnection &network );
+static void serve( int host_fd,
+		   Terminal::Complete &terminal,
+		   ServerConnection &network );
 
-int run_server( const char *desired_ip, const char *desired_port,
-		const string &command_path, char *command_argv[],
-		const int colors, bool verbose, bool with_motd );
+static int run_server( const char *desired_ip, const char *desired_port,
+		       const string &command_path, char *command_argv[],
+		       const int colors, bool verbose, bool with_motd );
 
 using namespace std;
 
-void print_usage( const char *argv0 )
+static void print_usage( const char *argv0 )
 {
   fprintf( stderr, "Usage: %s new [-s] [-v] [-i LOCALADDR] [-p PORT[:PORT2]] [-c COLORS] [-l NAME=VALUE] [-- COMMAND...]\n", argv0 );
 }
 
-void print_motd( void );
-void chdir_homedir( void );
-bool motd_hushed( void );
-void warn_unattached( const string & ignore_entry );
+static void print_motd( void );
+static void chdir_homedir( void );
+static bool motd_hushed( void );
+static void warn_unattached( const string & ignore_entry );
 
 /* Simple spinloop */
-void spin( void )
+static void spin( void )
 {
   static unsigned int spincount = 0;
   spincount++;
@@ -126,7 +126,7 @@ void spin( void )
   }
 }
 
-string get_SSH_IP( void )
+static string get_SSH_IP( void )
 {
   const char *SSH_CONNECTION = getenv( "SSH_CONNECTION" );
   if ( !SSH_CONNECTION ) { /* Older sshds don't set this */
@@ -200,7 +200,13 @@ int main( int argc, char *argv[] )
 	fatal_assert( desired_ip );
 	break;
       case 'c':
-	colors = myatoi( optarg );
+	try {
+	  colors = myatoi( optarg );
+	} catch ( const CryptoException & ) {
+	  fprintf( stderr, "%s: Bad number of colors (%s)\n", argv[ 0 ], optarg );
+	  print_usage( argv[ 0 ] );
+	  exit( 1 );
+	}
 	break;
       case 'v':
 	verbose = true;
@@ -227,13 +233,6 @@ int main( int argc, char *argv[] )
   }
 
   /* Sanity-check arguments */
-  if ( desired_ip
-       && ( strspn( desired_ip, "0123456789." ) != strlen( desired_ip ) ) ) {
-    fprintf( stderr, "%s: Bad IP address (%s)\n", argv[ 0 ], desired_ip );
-    print_usage( argv[ 0 ] );
-    exit( 1 );
-  }
-
   int dpl, dph;
   if ( desired_port && ! Connection::parse_portrange( desired_port, dpl, dph ) ) {
     fprintf( stderr, "%s: Bad UDP port range (%s)\n", argv[ 0 ], desired_port );
@@ -248,13 +247,17 @@ int main( int argc, char *argv[] )
   string shell_name;
   if ( !command_argv ) {
     /* get shell name */
-    struct passwd *pw = getpwuid( geteuid() );
-    if ( pw == NULL ) {
-      perror( "getpwuid" );
-      exit( 1 );
+    const char *shell = getenv( "SHELL" );
+    if ( shell == NULL ) {
+      struct passwd *pw = getpwuid( geteuid() );
+      if ( pw == NULL ) {
+	perror( "getpwuid" );
+	exit( 1 );
+      }
+      shell = pw->pw_shell;
     }
 
-    string shell_path( pw->pw_shell );
+    string shell_path( shell );
     if ( shell_path.empty() ) { /* empty shell means Bourne shell */
       shell_path = _PATH_BSHELL;
     }
@@ -317,26 +320,32 @@ int main( int argc, char *argv[] )
 
   try {
     return run_server( desired_ip, desired_port, command_path, command_argv, colors, verbose, with_motd );
-  } catch ( const Network::NetworkException& e ) {
-    fprintf( stderr, "Network exception: %s: %s\n",
-	     e.function.c_str(), strerror( e.the_errno ) );
+  } catch ( const Network::NetworkException &e ) {
+    fprintf( stderr, "Network exception: %s\n",
+	     e.what() );
     return 1;
-  } catch ( const Crypto::CryptoException& e ) {
+  } catch ( const Crypto::CryptoException &e ) {
     fprintf( stderr, "Crypto exception: %s\n",
-	     e.text.c_str() );
+	     e.what() );
     return 1;
   }
 }
 
-int run_server( const char *desired_ip, const char *desired_port,
-		const string &command_path, char *command_argv[],
-		const int colors, bool verbose, bool with_motd ) {
+static int run_server( const char *desired_ip, const char *desired_port,
+		       const string &command_path, char *command_argv[],
+		       const int colors, bool verbose, bool with_motd ) {
   /* get initial window size */
   struct winsize window_size;
-  if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &window_size ) < 0 ) {
-    perror( "ioctl TIOCGWINSZ" );
-    fprintf( stderr, "If running with ssh, please use ssh -t to provide a PTY.\n" );
-    exit( 1 );
+  if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &window_size ) < 0 ||
+       window_size.ws_col == 0 ||
+       window_size.ws_row == 0 ) {
+    fprintf( stderr, "Server started without pseudo-terminal. Opening 80x24 terminal.\n" );
+
+    /* Fill in sensible defaults. */
+    /* They will be overwritten by client on first connection. */
+    memset( &window_size, 0, sizeof( window_size ) );
+    window_size.ws_col = 80;
+    window_size.ws_row = 24;
   }
 
   /* open parser and terminal */
@@ -350,7 +359,7 @@ int run_server( const char *desired_ip, const char *desired_port,
     network->set_verbose();
   }
 
-  printf( "\nMOSH CONNECT %d %s\n", network->port(), network->get_key().c_str() );
+  printf( "\nMOSH CONNECT %s %s\n", network->port().c_str(), network->get_key().c_str() );
   fflush( stdout );
 
   /* don't let signals kill us */
@@ -361,13 +370,6 @@ int run_server( const char *desired_ip, const char *desired_port,
   fatal_assert( 0 == sigaction( SIGHUP, &sa, NULL ) );
   fatal_assert( 0 == sigaction( SIGPIPE, &sa, NULL ) );
 
-  struct termios child_termios;
-
-  /* Get terminal configuration */
-  if ( tcgetattr( STDIN_FILENO, &child_termios ) < 0 ) {
-    perror( "tcgetattr" );
-    exit( 1 );
-  }
 
   /* detach from terminal */
   pid_t the_pid = fork();
@@ -377,7 +379,7 @@ int run_server( const char *desired_ip, const char *desired_port,
     _exit( 0 );
   }
 
-  fprintf( stderr, "\nmosh-server (%s)\n", PACKAGE_STRING );
+  fprintf( stderr, "\nmosh-server (%s) [build %s]\n", PACKAGE_STRING, BUILD_VERSION );
   fprintf( stderr, "Copyright 2012 Keith Winstein <mosh-devel@mit.edu>\n" );
   fprintf( stderr, "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\nThis is free software: you are free to change and redistribute it.\nThere is NO WARRANTY, to the extent permitted by law.\n\n" );
 
@@ -385,13 +387,7 @@ int run_server( const char *desired_ip, const char *desired_port,
 
   int master;
 
-#ifdef HAVE_IUTF8
-  if ( !(child_termios.c_iflag & IUTF8) ) {
-    /* SSH should also convey IUTF8 across connection. */
-    //    fprintf( stderr, "Warning: Locale is UTF-8 but termios IUTF8 flag not set. Setting IUTF8 flag.\n" );
-    child_termios.c_iflag |= IUTF8;
-  }
-#else
+#ifndef HAVE_IUTF8
   fprintf( stderr, "\nWarning: termios IUTF8 flag not defined.\nCharacter-erase of multibyte character sequence\nprobably does not work properly on this platform.\n" );
 #endif /* HAVE_IUTF8 */
 
@@ -423,7 +419,7 @@ int run_server( const char *desired_ip, const char *desired_port,
   snprintf( utmp_entry, 64, "mosh [%d]", getpid() );
 
   /* Fork child process */
-  pid_t child = forkpty( &master, NULL, &child_termios, &window_size );
+  pid_t child = forkpty( &master, NULL, NULL, &window_size );
 
   if ( child == -1 ) {
     perror( "forkpty" );
@@ -444,6 +440,22 @@ int run_server( const char *desired_ip, const char *desired_port,
     /* close server-related file descriptors */
     delete network;
 
+    /* set IUTF8 if available */
+#ifdef HAVE_IUTF8
+    struct termios child_termios;
+    if ( tcgetattr( STDIN_FILENO, &child_termios ) < 0 ) {
+      perror( "tcgetattr" );
+      exit( 1 );
+    }
+
+    child_termios.c_iflag |= IUTF8;
+
+    if ( tcsetattr( STDIN_FILENO, TCSANOW, &child_termios ) < 0 ) {
+      perror( "tcsetattr" );
+      exit( 1 );
+    }
+#endif /* HAVE_IUTF8 */
+
     /* set TERM */
     const char default_term[] = "xterm";
     const char color_term[] = "xterm-256color";
@@ -488,12 +500,12 @@ int run_server( const char *desired_ip, const char *desired_port,
 
     try {
       serve( master, terminal, *network );
-    } catch ( const Network::NetworkException& e ) {
-      fprintf( stderr, "Network exception: %s: %s\n",
-	       e.function.c_str(), strerror( e.the_errno ) );
-    } catch ( const Crypto::CryptoException& e ) {
+    } catch ( const Network::NetworkException &e ) {
+      fprintf( stderr, "Network exception: %s\n",
+	       e.what() );
+    } catch ( const Crypto::CryptoException &e ) {
       fprintf( stderr, "Crypto exception: %s\n",
-	       e.text.c_str() );
+	       e.what() );
     }
 
     #ifdef HAVE_UTEMPTER
@@ -513,7 +525,7 @@ int run_server( const char *desired_ip, const char *desired_port,
   return 0;
 }
 
-void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network )
+static void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network )
 {
   /* prepare to poll for events */
   Select &sel = Select::get_instance();
@@ -525,8 +537,8 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
   #ifdef HAVE_UTEMPTER
   bool connected_utmp = false;
 
-  struct in_addr saved_addr;
-  saved_addr.s_addr = 0;
+  Addr saved_addr;
+  socklen_t saved_addr_len = 0;
   #endif
 
   while ( 1 ) {
@@ -573,8 +585,9 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	  us.apply_string( network.get_remote_diff() );
 	  /* apply userstream to terminal */
 	  for ( size_t i = 0; i < us.size(); i++ ) {
-	    terminal_to_host += terminal.act( us.get_action( i ) );
-	    if ( typeid( *us.get_action( i ) ) == typeid( Parser::Resize ) ) {
+	    const Parser::Action *action = us.get_action( i );
+	    terminal_to_host += terminal.act( action );
+	    if ( typeid( *action ) == typeid( Parser::Resize ) ) {
 	      /* tell child process of resize */
 	      const Parser::Resize *res = static_cast<const Parser::Resize *>( us.get_action( i ) );
 	      struct winsize window_size;
@@ -609,13 +622,24 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	  #ifdef HAVE_UTEMPTER
 	  /* update utmp entry if we have become "connected" */
 	  if ( (!connected_utmp)
-	       || ( saved_addr.s_addr != network.get_remote_ip().s_addr ) ) {
+	       || saved_addr_len != network.get_remote_addr_len()
+	       || memcmp( &saved_addr, &network.get_remote_addr(),
+			  saved_addr_len ) != 0 ) {
 	    utempter_remove_record( host_fd );
 
-	    saved_addr = network.get_remote_ip();
+	    saved_addr = network.get_remote_addr();
+	    saved_addr_len = network.get_remote_addr_len();
+
+	    char host[ NI_MAXHOST ];
+	    int errcode = getnameinfo( &saved_addr.sa, saved_addr_len,
+				       host, sizeof( host ), NULL, 0,
+				       NI_NUMERICHOST );
+	    if ( errcode != 0 ) {
+	      throw NetworkException( std::string( "serve: getnameinfo: " ) + gai_strerror( errcode ), 0 );
+	    }
 
 	    char tmp[ 64 ];
-	    snprintf( tmp, 64, "%s via mosh [%d]", inet_ntoa( saved_addr ), getpid() );
+	    snprintf( tmp, 64, "%s via mosh [%d]", host, getpid() );
 	    utempter_add_record( host_fd, tmp );
 
 	    connected_utmp = true;
@@ -713,21 +737,21 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
       }
 
       network.tick();
-    } catch ( const Network::NetworkException& e ) {
-      fprintf( stderr, "%s: %s\n", e.function.c_str(), strerror( e.the_errno ) );
+    } catch ( const Network::NetworkException &e ) {
+      fprintf( stderr, "%s\n", e.what() );
       spin();
-    } catch ( const Crypto::CryptoException& e ) {
+    } catch ( const Crypto::CryptoException &e ) {
       if ( e.fatal ) {
         throw;
       } else {
-        fprintf( stderr, "Crypto exception: %s\n", e.text.c_str() );
+        fprintf( stderr, "Crypto exception: %s\n", e.what() );
       }
     }
   }
 }
 
 /* OpenSSH prints the motd on startup, so we will too */
-void print_motd( void )
+static void print_motd( void )
 {
   FILE *motd = fopen( "/etc/motd", "r" );
   if ( !motd ) {
@@ -751,51 +775,42 @@ void print_motd( void )
   fclose( motd );
 }
 
-void chdir_homedir( void )
+static void chdir_homedir( void )
 {
-  struct passwd *pw = getpwuid( geteuid() );
-  if ( pw == NULL ) {
-    perror( "getpwuid" );
-    return; /* non-fatal */
+  const char *home = getenv( "HOME" );
+  if ( home == NULL ) {
+    struct passwd *pw = getpwuid( geteuid() );
+    if ( pw == NULL ) {
+      perror( "getpwuid" );
+      return; /* non-fatal */
+    }
+    home = pw->pw_dir;
   }
 
-  if ( chdir( pw->pw_dir ) < 0 ) {
+  if ( chdir( home ) < 0 ) {
     perror( "chdir" );
   }
 
-  if ( setenv( "PWD", pw->pw_dir, 1 ) < 0 ) {
+  if ( setenv( "PWD", home, 1 ) < 0 ) {
     perror( "setenv" );
   }
 }
 
-bool motd_hushed( void )
+static bool motd_hushed( void )
 {
   /* must be in home directory already */
   struct stat buf;
   return (0 == lstat( ".hushlogin", &buf ));
 }
 
-bool device_exists( const char *ut_line )
+static bool device_exists( const char *ut_line )
 {
   string device_name = string( "/dev/" ) + string( ut_line );
   struct stat buf;
   return (0 == lstat( device_name.c_str(), &buf ));
 }
 
-string mosh_read_line( FILE *file )
-{
-  string ret;
-  while ( !feof( file ) ) {
-    char next = getc( file );
-    if ( next == '\n' ) {
-      return ret;
-    }
-    ret.push_back( next );
-  }
-  return ret;
-}
-
-void warn_unattached( const string & ignore_entry )
+static void warn_unattached( const string & ignore_entry )
 {
 #ifdef HAVE_UTMPX_H
   /* get username */
