@@ -41,6 +41,8 @@
 #include <math.h>
 #include <vector>
 #include <assert.h>
+#include <exception>
+#include <string.h>
 
 #include "crypto.h"
 
@@ -53,12 +55,18 @@ namespace Network {
   uint16_t timestamp16( void );
   uint16_t timestamp_diff( uint16_t tsnew, uint16_t tsold );
 
-  class NetworkException {
+  class NetworkException : public std::exception {
   public:
     string function;
     int the_errno;
-    NetworkException( string s_function, int s_errno ) : function( s_function ), the_errno( s_errno ) {}
-    NetworkException() : function( "<none>" ), the_errno( 0 ) {}
+  private:
+    string my_what;
+  public:
+    NetworkException( string s_function="<none>", int s_errno=0)
+      : function( s_function ), the_errno( s_errno ),
+        my_what(function + ": " + strerror(the_errno)) {}
+    const char *what() const throw () { return my_what.c_str(); }
+    ~NetworkException() throw () {}
   };
 
   enum Direction {
@@ -84,6 +92,13 @@ namespace Network {
     string tostring( Session *session );
   };
 
+  union Addr {
+    struct sockaddr sa;
+    struct sockaddr_in sin;
+    struct sockaddr_in6 sin6;
+    struct sockaddr_storage ss;
+  };
+
   class Connection {
   private:
     static const int DEFAULT_SEND_MTU = 1300;
@@ -101,7 +116,7 @@ namespace Network {
 
     static const int CONGESTION_TIMESTAMP_PENALTY = 500; /* ms */
 
-    static bool try_bind( int socket, uint32_t addr, int port_low, int port_high );
+    bool try_bind( const char *addr, int port_low, int port_high );
 
     class Socket
     {
@@ -110,7 +125,7 @@ namespace Network {
 
     public:
       int fd( void ) const { return _fd; }
-      Socket();
+      Socket( int family );
       ~Socket();
 
       Socket( const Socket & other );
@@ -119,7 +134,8 @@ namespace Network {
 
     std::deque< Socket > socks;
     bool has_remote_addr;
-    struct sockaddr_in remote_addr;
+    Addr remote_addr;
+    socklen_t remote_addr_len;
 
     bool server;
 
@@ -161,21 +177,22 @@ namespace Network {
 
   public:
     Connection( const char *desired_ip, const char *desired_port ); /* server */
-    Connection( const char *key_str, const char *ip, int port ); /* client */
+    Connection( const char *key_str, const char *ip, const char *port ); /* client */
 
     void send( string s );
     string recv( void );
     const std::vector< int > fds( void ) const;
     int get_MTU( void ) const { return MTU; }
 
-    int port( void ) const;
+    std::string port( void ) const;
     string get_key( void ) const { return key.printable_key(); }
     bool get_has_remote_addr( void ) const { return has_remote_addr; }
 
     uint64_t timeout( void ) const;
     double get_SRTT( void ) const { return SRTT; }
 
-    const struct in_addr & get_remote_ip( void ) const { return remote_addr.sin_addr; }
+    const Addr &get_remote_addr( void ) const { return remote_addr; }
+    socklen_t get_remote_addr_len( void ) const { return remote_addr_len; }
 
     const NetworkException *get_send_exception( void ) const
     {
