@@ -56,7 +56,7 @@ bool verbose = false;
 
 #define NONCE_FMT "%016"PRIx64
 
-std::string random_payload( void ) {
+static std::string random_payload( void ) {
   const size_t len = prng.uint32() % MESSAGE_SIZE_MAX;
   char *buf = new char[len];
   prng.fill( buf, len );
@@ -66,13 +66,13 @@ std::string random_payload( void ) {
   return payload;
 }
 
-void test_bad_decrypt( Session &decryption_session ) {
+static void test_bad_decrypt( Session &decryption_session ) {
   std::string bad_ct = random_payload();
 
   bool got_exn = false;
   try {
     decryption_session.decrypt( bad_ct );
-  } catch ( const CryptoException& e ) {
+  } catch ( const CryptoException &e ) {
     got_exn = true;
 
     /* The "bad decrypt" exception needs to be non-fatal, otherwise we are
@@ -87,7 +87,7 @@ void test_bad_decrypt( Session &decryption_session ) {
 }
 
 /* Generate a single key and initial nonce, then perform some encryptions. */
-void test_one_session( void ) {
+static void test_one_session( void ) {
   Base64Key key;
   Session encryption_session( key );
   Session decryption_session( key );
@@ -142,9 +142,9 @@ int main( int argc, char *argv[] ) {
   for ( size_t i=0; i<NUM_SESSIONS; i++ ) {
     try {
       test_one_session();
-    } catch ( const CryptoException& e ) {
+    } catch ( const CryptoException &e ) {
       fprintf( stderr, "Crypto exception: %s\r\n",
-               e.text.c_str() );
+               e.what() );
       fatal_assert( false );
     }
   }
